/* Target-dependent code for Solaris2 running on Sparc's, for GDB.

   Copyright 2002 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "defs.h"
#include "target.h"

#include "sparc-tdep.h"

/* There are two different signal handler trampolines in Solaris2.  */
static int
sparc_sol_pc_in_sigtramp (CORE_ADDR pc, char *name)
{
  return (name
	  && (! strcmp ("sigacthandler", name)
	      || ! strcmp ("ucbsigvechandler", name)));
}

extern CORE_ADDR sparc_frame_saved_pc (struct frame_info *);

static CORE_ADDR
sparc_sol_frame_saved_pc (struct frame_info *frame)
{
  if (frame->signal_handler_caller)
    {
      unsigned long sigcontext_pc_offset;
      CORE_ADDR sigcontext_addr, saved_pc_offset;
      char *scbuf;
      char *name = NULL;

      /* The signal handler gets a pointer to an ucontext as third argument
	 if it is called from sigacthandler.  This is the offset to the saved
	 PC within it.  */
      if (GDB_TARGET_IS_SPARC64)
	saved_pc_offset = 88;
      else
	saved_pc_offset = 44;

      scbuf = alloca (TARGET_PTR_BIT / HOST_CHAR_BIT);

      /* Solaris2 ucbsigvechandler passes a pointer to a sigcontext
         as the third parameter.  The offset to the saved pc is 12.  */
      find_pc_partial_function (frame->pc, &name,
				(CORE_ADDR *) NULL, (CORE_ADDR *) NULL);
      if (name && STREQ (name, "ucbsigvechandler"))
	{
	  if (GDB_TARGET_IS_SPARC64)
	    saved_pc_offset = 24;
	  else
	    saved_pc_offset = 12;
	}

      /* The sigcontext address is contained in register O2.  */
      get_saved_register (buf, (int *) NULL, (CORE_ADDR *) NULL,
			  frame, O0_REGNUM + 2, (enum lval_type *) NULL);
      sigcontext_addr = extract_address (buf, REGISTER_RAW_SIZE (O0_REGNUM + 2));

      /* Don't cause a memory_error when accessing sigcontext in case the
         stack layout has changed or the stack is corrupt.  */
      target_read_memory (sigcontext_addr + saved_pc_offset,
			  scbuf, TARGET_PTR_BIT / HOST_CHAR_BIT);
      return extract_address (scbuf, sizeof (scbuf));
    }

  return sparc_frame_saved_pc (frame);
}

static void
sparc_sol_gdbarch_os_init(struct gdbarch_info info, struct gdbarch *gdbarch)
{
  set_gdbarch_pc_in_sigtramp (gdbarch, sparc_sol_pc_in_sigtramp);
  set_gdbarch_frame_saved_pc (gdbarch, sparc_sol_frame_saved_pc);

  /* With Sol2 it is no longer necessary to enable software single-step,
     since the /proc interface can take care of it for us in hardware.  */
  set_gdbarch_software_single_step (gdbarch, 0);
}

void
_initialize_sparc_sol_tdep (void)
{
  sparc_gdbarch_register_os_abi (SPARC_OS_SOLARIS, sparc_sol_gdbarch_os_init);
}
