/* Target-dependent code for SunOS4 running on Sparc's, for GDB.

   Copyright 2002 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "defs.h"
#include "target.h"
#include "symtab.h"

#include "sparc-tdep.h"

static int
sparc_sunos_get_longjmp_target (CORE_ADDR *pc)
{
  CORE_ADDR jb_addr, jb_pc;
  char *buf;

  buf = alloca (SPARC_INTREG_SIZE);

  jb_pc = 3;
  jb_addr = read_register (O0_REGNUM);
  if (target_read_memory (jb_addr + jb_pc * SPARC_INTREG_SIZE, buf,
			  SPARC_INTREG_SIZE))
    return 0;

  *pc = extract_address (buf, SPARC_INTREG_SIZE);

  return 1;
}

/* If pc is in a shared library trampoline, return its target.
   The SunOs 4.x linker rewrites the jump table entries for PIC
   compiled modules in the main executable to bypass the dynamic linker
   with jumps of the form
   sethi %hi(addr),%g1
   jmp %g1+%lo(addr)
   and removes the corresponding jump table relocation entry in the
   dynamic relocations.
   find_solib_trampoline_target relies on the presence of the jump
   table relocation entry, so we have to detect these jump instructions
   by hand.  */

static CORE_ADDR
sparc_sunos_skip_trampoline_code (CORE_ADDR pc)
{
  unsigned long insn1;
  char buf[4];
  int err;

  err = target_read_memory (pc, buf, 4);
  insn1 = extract_unsigned_integer (buf, 4);
  if (err == 0 && (insn1 & 0xffc00000) == 0x03000000)
    {
      unsigned long insn2;

      err = target_read_memory (pc + 4, buf, 4);
      insn2 = extract_unsigned_integer (buf, 4);
      if (err == 0 && (insn2 & 0xffffe000) == 0x81c06000)
	{
	  CORE_ADDR target_pc = (insn1 & 0x3fffff) << 10;
	  int delta = insn2 & 0x1fff;

	  /* Sign extend the displacement.  */
	  if (delta & 0x1000)
	    delta |= ~0x1fff;
	  return target_pc + delta;
	}
    }
  return find_solib_trampoline_target (pc);
}

static void
sparc_sunos_gdbarch_os_init(struct gdbarch_info info, struct gdbarch *gdbarch)
{
  set_gdbarch_get_longjmp_target (gdbarch, sparc_sunos_get_longjmp_target);
  set_gdbarch_skip_trampoline_code (gdbarch, saprc_sunos_skip_trampoline_code);
}

void
_initialize_sparc_sunos_tdep (void)
{
  sparc_gdbarch_register_os_abi (SPARC_OS_SUNOS, sparc_sunos_gdbarch_os_init);
}
